@@ -22,7 +22,7 @@ use std::process::Stdio;
 #[derive(clap::Args, Debug, Clone)]
 #[clap(author)]
 pub(crate) struct Start {
-    /// The Postgres version to stop (pg13, pg14, pg15, pg16, pg17, or all)
+    /// The Postgres version to stop (pg13, pg14, pg15, pg16, pg17, pg18, or all)
     #[clap(env = "PG_VERSION")]
     pg_version: Option<String>,
     #[clap(from_global, action = ArgAction::Count)]
@@ -79,7 +79,7 @@ impl CommandExecute for Start {
 }
 
 pub(crate) fn collect_postgresql_conf_settings(
-    settings: &Vec<String>,
+    settings: &[String],
 ) -> eyre::Result<HashMap<String, String>> {
     settings
         .iter()
