@@ -22,6 +22,8 @@ use std::str::FromStr;
 use thiserror::Error;
 use url::Url;
 
+mod decoding;
+
 pub mod cargo;
 
 pub static BASE_POSTGRES_PORT_NO: u16 = 28800;
@@ -61,6 +63,8 @@ pub fn get_c_locale_flags() -> &'static [&'static str] {
 mod path_methods;
 pub use path_methods::{get_target_dir, prefix_path};
 
+use crate::decoding::decode_from_bytes;
+
 #[derive(Copy, Clone, Debug, Eq, PartialEq)]
 pub enum PgMinorVersion {
     Latest,
@@ -470,7 +474,7 @@ impl PgConfig {
             });
 
             match Command::new(&pg_config).arg(arg).output() {
-                Ok(output) => Ok(String::from_utf8(output.stdout).unwrap().trim().to_string()),
+                Ok(output) => Ok(decode_from_bytes(&output.stdout).trim().to_string()),
                 Err(e) => match e.kind() {
                     ErrorKind::NotFound => Err(e).wrap_err_with(|| {
                         let pg_config_str = pg_config.display().to_string();
@@ -707,6 +711,16 @@ pub fn SUPPORTED_VERSIONS() -> Vec<PgVersion> {
         PgVersion::new(15, PgMinorVersion::Latest, None),
         PgVersion::new(16, PgMinorVersion::Latest, None),
         PgVersion::new(17, PgMinorVersion::Latest, None),
+        PgVersion::new(
+            18,
+            PgMinorVersion::Beta(1),
+            Some(
+                Url::parse(
+                    "https://ftp.postgresql.org/pub/source/v18beta1/postgresql-18beta1.tar.bz2",
+                )
+                .expect("malformed pg18beta1 url"),
+            ),
+        ),
     ]
 }
 
@@ -767,8 +781,8 @@ pub fn createdb(
         return Err(eyre!(
             "problem running createdb: {}\n\n{}{}",
             command_str,
-            String::from_utf8(output.stdout).unwrap(),
-            String::from_utf8(output.stderr).unwrap()
+            decode_from_bytes(&output.stdout),
+            decode_from_bytes(&output.stderr)
         ));
     }
 
@@ -827,8 +841,8 @@ pub fn dropdb(
         return Err(eyre!(
             "problem running dropdb: {}\n\n{}{}",
             command_str,
-            String::from_utf8(output.stdout).unwrap(),
-            String::from_utf8(output.stderr).unwrap()
+            decode_from_bytes(&output.stdout),
+            decode_from_bytes(&output.stderr)
         ));
     }
 
@@ -844,12 +858,12 @@ fn does_db_exist(pg_config: &PgConfig, dbname: &str) -> eyre::Result<bool> {
         .arg(pg_config.host())
         .arg("-p")
         .arg(pg_config.port()?.to_string())
-        .arg("template1")
         .arg("-c")
         .arg(format!(
             "select count(*) from pg_database where datname = '{}';",
             dbname.replace('\'', "''")
         ))
+        .arg("template1")
         .stdout(Stdio::piped())
         .stderr(Stdio::piped());
 
@@ -861,11 +875,11 @@ fn does_db_exist(pg_config: &PgConfig, dbname: &str) -> eyre::Result<bool> {
             "problem checking if database '{}' exists: {}\n\n{}{}",
             dbname,
             command_str,
-            String::from_utf8(output.stdout).unwrap(),
-            String::from_utf8(output.stderr).unwrap()
+            decode_from_bytes(&output.stdout),
+            decode_from_bytes(&output.stderr)
         ))
     } else {
-        let count = i32::from_str(String::from_utf8(output.stdout).unwrap().trim())
+        let count = i32::from_str(decode_from_bytes(&output.stdout).trim())
             .wrap_err("result is not a number")?;
         Ok(count > 0)
     }
