@@ -55,6 +55,8 @@ pub struct PostgresTypeDerive {
     generics: Generics,
     in_fn: Ident,
     out_fn: Ident,
+    receive_fn: Option<Ident>,
+    send_fn: Option<Ident>,
     to_sql_config: ToSqlConfig,
     alignment: Alignment,
 }
@@ -65,17 +67,29 @@ impl PostgresTypeDerive {
         generics: Generics,
         in_fn: Ident,
         out_fn: Ident,
+        receive_fn: Option<Ident>,
+        send_fn: Option<Ident>,
         to_sql_config: ToSqlConfig,
         alignment: Alignment,
     ) -> Result<CodeEnrichment<Self>, syn::Error> {
         if !to_sql_config.overrides_default() {
             crate::ident_is_acceptable_to_postgres(&name)?;
         }
-        Ok(CodeEnrichment(Self { generics, name, in_fn, out_fn, to_sql_config, alignment }))
+        Ok(CodeEnrichment(Self {
+            generics,
+            name,
+            in_fn,
+            out_fn,
+            receive_fn,
+            send_fn,
+            to_sql_config,
+            alignment,
+        }))
     }
 
     pub fn from_derive_input(
         derive_input: DeriveInput,
+        pg_binary_protocol: bool,
     ) -> Result<CodeEnrichment<Self>, syn::Error> {
         match derive_input.data {
             syn::Data::Struct(_) | syn::Data::Enum(_) => {}
@@ -93,12 +107,26 @@ impl PostgresTypeDerive {
             &format!("{}_out", derive_input.ident).to_lowercase(),
             derive_input.ident.span(),
         );
+        let funcname_receive = (pg_binary_protocol).then(|| {
+            Ident::new(
+                &format!("{}_recv", derive_input.ident).to_lowercase(),
+                derive_input.ident.span(),
+            )
+        });
+        let funcname_send = (pg_binary_protocol).then(|| {
+            Ident::new(
+                &format!("{}_send", derive_input.ident).to_lowercase(),
+                derive_input.ident.span(),
+            )
+        });
         let alignment = Alignment::from_attributes(derive_input.attrs.as_slice())?;
         Self::new(
             derive_input.ident,
             derive_input.generics,
             funcname_in,
             funcname_out,
+            funcname_receive,
+            funcname_send,
             to_sql_config,
             alignment,
         )
@@ -129,6 +157,40 @@ impl ToEntityGraphTokens for PostgresTypeDerive {
 
         let in_fn = &self.in_fn;
         let out_fn = &self.out_fn;
+        let stringify_receive_fn = self
+            .receive_fn
+            .as_ref()
+            .map(|f| quote! { Some(stringify!(#f)) })
+            .unwrap_or_else(|| quote! { None });
+        let stringify_send_fn = self
+            .send_fn
+            .as_ref()
+            .map(|f| quote! { Some(stringify!(#f)) })
+            .unwrap_or_else(|| quote! { None });
+        let receive_fn_module_path = self
+            .receive_fn
+            .as_ref()
+            .map(|f| {
+                quote! {Some({
+                    let in_fn = stringify!(#f);
+                    let mut path_items: Vec<_> = in_fn.split("::").collect();
+                    let _ = path_items.pop(); // Drop the one we don't want.
+                    path_items.join("::")
+                })}
+            })
+            .unwrap_or_else(|| quote! { None });
+        let send_fn_module_path = self
+            .send_fn
+            .as_ref()
+            .map(|f| {
+                quote! {Some({
+                    let out_fn = stringify!(#f);
+                    let mut path_items: Vec<_> = out_fn.split("::").collect();
+                    let _ = path_items.pop(); // Drop the one we don't want.
+                    path_items.join("::")
+                })}
+            })
+            .unwrap_or_else(|| quote! { None });
 
         let sql_graph_entity_fn_name = format_ident!("__pgrx_internals_type_{}", self.name);
 
@@ -199,6 +261,10 @@ impl ToEntityGraphTokens for PostgresTypeDerive {
                         let _ = path_items.pop(); // Drop the one we don't want.
                         path_items.join("::")
                     },
+                    receive_fn: #stringify_receive_fn,
+                    receive_fn_module_path: #receive_fn_module_path,
+                    send_fn: #stringify_send_fn,
+                    send_fn_module_path: #send_fn_module_path,
                     to_sql_config: #to_sql_config,
                     alignment: #alignment,
                 };
@@ -213,10 +279,26 @@ impl ToRustCodeTokens for PostgresTypeDerive {}
 impl Parse for CodeEnrichment<PostgresTypeDerive> {
     fn parse(input: ParseStream) -> Result<Self, syn::Error> {
         let ItemStruct { attrs, ident, generics, .. } = input.parse()?;
+
+        let pg_binary_protocol = attrs.iter().any(|a| a.path().is_ident("pg_binary_protocol"));
+
         let to_sql_config = ToSqlConfig::from_attributes(attrs.as_slice())?.unwrap_or_default();
-        let in_fn = Ident::new(&format!("{}_in", ident).to_lowercase(), ident.span());
-        let out_fn = Ident::new(&format!("{}_out", ident).to_lowercase(), ident.span());
+        let in_fn = Ident::new(&format!("{ident}_in").to_lowercase(), ident.span());
+        let out_fn = Ident::new(&format!("{ident}_out").to_lowercase(), ident.span());
+        let receive_fn = (pg_binary_protocol)
+            .then(|| Ident::new(&format!("{ident}_recv").to_lowercase(), ident.span()));
+        let send_fn = (pg_binary_protocol)
+            .then(|| Ident::new(&format!("{ident}_send").to_lowercase(), ident.span()));
         let alignment = Alignment::from_attributes(attrs.as_slice())?;
-        PostgresTypeDerive::new(ident, generics, in_fn, out_fn, to_sql_config, alignment)
+        PostgresTypeDerive::new(
+            ident,
+            generics,
+            in_fn,
+            out_fn,
+            receive_fn,
+            send_fn,
+            to_sql_config,
+            alignment,
+        )
     }
 }
