@@ -407,7 +407,7 @@ fn maybe_make_pgdata<P: AsRef<Path>>(pgdata: P) -> eyre::Result<bool> {
 
         let mut mkdir = sudo_command(&runas);
         mkdir.arg("mkdir").arg("-p").arg(pgdata).stdout(Stdio::piped()).stderr(Stdio::piped());
-        let command_str = format!("{:?}", mkdir);
+        let command_str = format!("{mkdir:?}");
         println!("{} {}", "     Running".bold().green(), command_str);
         let child = mkdir.spawn()?;
         let output = child.wait_with_output()?;
@@ -584,9 +584,11 @@ fn start_pg(loglines: LogLines) -> eyre::Result<String> {
         #[inline]
         fn accept_envar(var: &str) -> bool {
             // taken from https://doc.rust-lang.org/cargo/reference/environment-variables.html
+            // and https://releases.llvm.org/12.0.1/tools/clang/docs/SourceBasedCodeCoverage.html
             var.starts_with("CARGO")
                 || var.starts_with("RUST")
                 || var.starts_with("DEP_")
+                || var.starts_with("LLVM_")
                 || ["OUT_DIR", "TARGET", "HOST", "NUM_JOBS", "OPT_LEVEL", "DEBUG", "PROFILE"]
                     .contains(&var)
         }
@@ -845,7 +847,7 @@ fn get_extension_name() -> eyre::Result<String> {
     // https://github.com/rust-lang/cargo/issues/45
     let path = PathBuf::from(dir).join("Cargo.toml");
     let name = pgrx_pg_config::cargo::read_manifest(path)?.lib_name()?;
-    Ok(name.replace('-', "_"))
+    Ok(name)
 }
 
 fn get_pgdata_path() -> eyre::Result<PathBuf> {
