@@ -1,3 +1,4 @@
+use std::marker::PhantomData;
 //LICENSE Portions Copyright 2019-2021 ZomboDB, LLC.
 //LICENSE
 //LICENSE Portions Copyright 2021-2023 Technology Concepts & Design, Inc.
@@ -19,6 +20,7 @@ fn extern_func() -> bool {
 // Uncommenting the line below will make it fail to compile
 // #[no_mangle]
 extern "C-unwind" fn extern_func_impl<T>() -> bool {
+    let _ = PhantomData::<T>;
     true
 }
 
@@ -34,8 +36,9 @@ extern "C-unwind" fn extern_func_impl_1() -> bool {
 // and [no_mangle]
 #[pg_guard]
 #[no_mangle]
-#[allow(unused_lifetimes)]
+#[allow(unused_lifetimes, clippy::extra_unused_lifetimes)]
 extern "C-unwind" fn extern_func_impl_2<'a>() -> bool {
+    let _ = PhantomData::<&'a ()>;
     true
 }
 
