@@ -76,7 +76,8 @@ impl BackgroundWorker {
             feature = "pg14",
             feature = "pg15",
             feature = "pg16",
-            feature = "pg17"
+            feature = "pg17",
+            feature = "pg18"
         ))]
         const LEN: usize = 96;
 
@@ -197,7 +198,8 @@ impl BackgroundWorker {
                 feature = "pg14",
                 feature = "pg15",
                 feature = "pg16",
-                feature = "pg17"
+                feature = "pg17",
+                feature = "pg18"
             ))]
             pg_sys::BackgroundWorkerInitializeConnection(db, user, 0);
         };
@@ -216,16 +218,52 @@ impl BackgroundWorker {
         unsafe {
             assert!(!pg_sys::MyBgworkerEntry.is_null(), "BackgroundWorker associated functions can only be called from a registered background worker");
             if wake.contains(SignalWakeFlags::SIGHUP) {
+                #[cfg(any(
+                    feature = "pg13",
+                    feature = "pg14",
+                    feature = "pg15",
+                    feature = "pg16",
+                    feature = "pg17"
+                ))]
                 pg_sys::pqsignal(pg_sys::SIGHUP as i32, Some(worker_spi_sighup));
+                #[cfg(feature = "pg18")]
+                pg_sys::pqsignal_be(pg_sys::SIGHUP as i32, Some(worker_spi_sighup));
             }
             if wake.contains(SignalWakeFlags::SIGTERM) {
+                #[cfg(any(
+                    feature = "pg13",
+                    feature = "pg14",
+                    feature = "pg15",
+                    feature = "pg16",
+                    feature = "pg17"
+                ))]
                 pg_sys::pqsignal(pg_sys::SIGTERM as i32, Some(worker_spi_sigterm));
+                #[cfg(feature = "pg18")]
+                pg_sys::pqsignal_be(pg_sys::SIGTERM as i32, Some(worker_spi_sigterm));
             }
             if wake.contains(SignalWakeFlags::SIGINT) {
+                #[cfg(any(
+                    feature = "pg13",
+                    feature = "pg14",
+                    feature = "pg15",
+                    feature = "pg16",
+                    feature = "pg17"
+                ))]
                 pg_sys::pqsignal(pg_sys::SIGINT as i32, Some(worker_spi_sigint));
+                #[cfg(feature = "pg18")]
+                pg_sys::pqsignal_be(pg_sys::SIGINT as i32, Some(worker_spi_sigint));
             }
             if wake.contains(SignalWakeFlags::SIGCHLD) {
+                #[cfg(any(
+                    feature = "pg13",
+                    feature = "pg14",
+                    feature = "pg15",
+                    feature = "pg16",
+                    feature = "pg17"
+                ))]
                 pg_sys::pqsignal(pg_sys::SIGCHLD as i32, Some(worker_spi_sigchld));
+                #[cfg(feature = "pg18")]
+                pg_sys::pqsignal_be(pg_sys::SIGCHLD as i32, Some(worker_spi_sigchld));
             }
             pg_sys::BackgroundWorkerUnblockSignals();
         }
@@ -253,12 +291,13 @@ impl BackgroundWorker {
 
 unsafe extern "C-unwind" fn worker_spi_sighup(_signal_args: i32) {
     GOT_SIGHUP.store(true, Ordering::SeqCst);
-    pg_sys::ProcessConfigFile(pg_sys::GucContext::PGC_SIGHUP);
+    (&raw mut pg_sys::ConfigReloadPending).write_volatile(1);
     pg_sys::SetLatch(pg_sys::MyLatch);
 }
 
 unsafe extern "C-unwind" fn worker_spi_sigterm(_signal_args: i32) {
     GOT_SIGTERM.store(true, Ordering::SeqCst);
+    (&raw mut pg_sys::ShutdownRequestPending).write_volatile(1);
     pg_sys::SetLatch(pg_sys::MyLatch);
 }
 
@@ -272,6 +311,11 @@ unsafe extern "C-unwind" fn worker_spi_sigchld(_signal_args: i32) {
     pg_sys::SetLatch(pg_sys::MyLatch);
 }
 
+/// Indicates that a [`DynamicBackgroundWorker`] could not be loaded.
+/// There's no reason why.
+#[derive(Debug, Clone, Copy)]
+pub struct DynamicBackgroundWorkerLoadError;
+
 /// Dynamic background worker handle
 pub struct DynamicBackgroundWorker {
     handle: *mut pg_sys::BackgroundWorkerHandle,
@@ -599,7 +643,7 @@ impl BackgroundWorkerBuilder {
 
     /// Once properly configured, call `load_dynamic()` to get the BackgroundWorker registered and started dynamically.
     /// Start up might fail, e.g. if max_worker_processes is exceeded. In that case an Err is returned.
-    pub fn load_dynamic(self) -> Result<DynamicBackgroundWorker, ()> {
+    pub fn load_dynamic(self) -> Result<DynamicBackgroundWorker, DynamicBackgroundWorkerLoadError> {
         let mut bgw: pg_sys::BackgroundWorker = (&self).into();
         let mut handle: *mut pg_sys::BackgroundWorkerHandle = null_mut();
 
@@ -607,7 +651,7 @@ impl BackgroundWorkerBuilder {
         let success = unsafe { pg_sys::RegisterDynamicBackgroundWorker(&mut bgw, &mut handle) };
 
         if !success {
-            Err(())
+            Err(DynamicBackgroundWorkerLoadError)
         } else {
             Ok(DynamicBackgroundWorker { handle, notify_pid: bgw.bgw_notify_pid })
         }
@@ -619,14 +663,7 @@ impl BackgroundWorkerBuilder {
 /// the builder is useful for building this structure.
 impl<'a> From<&'a BackgroundWorkerBuilder> for pg_sys::BackgroundWorker {
     fn from(builder: &'a BackgroundWorkerBuilder) -> Self {
-        #[cfg(any(
-            feature = "pg13",
-            feature = "pg14",
-            feature = "pg15",
-            feature = "pg16",
-            feature = "pg17"
-        ))]
-        let bgw = pg_sys::BackgroundWorker {
+        pg_sys::BackgroundWorker {
             bgw_name: RpgffiChar::from(&builder.bgw_name[..]).0,
             bgw_type: RpgffiChar::from(&builder.bgw_type[..]).0,
             bgw_flags: builder.bgw_flags.bits(),
@@ -636,12 +673,12 @@ impl<'a> From<&'a BackgroundWorkerBuilder> for pg_sys::BackgroundWorker {
                 Some(d) => d.as_secs() as i32,
             },
             bgw_library_name: {
-                #[cfg(not(feature = "pg17"))]
+                #[cfg(not(any(feature = "pg17", feature = "pg18")))]
                 {
                     RpgffiChar::from(&builder.bgw_library_name[..]).0
                 }
 
-                #[cfg(feature = "pg17")]
+                #[cfg(any(feature = "pg17", feature = "pg18"))]
                 {
                     RpgffiChar1024::from(&builder.bgw_library_name[..]).0
                 }
@@ -650,9 +687,7 @@ impl<'a> From<&'a BackgroundWorkerBuilder> for pg_sys::BackgroundWorker {
             bgw_main_arg: builder.bgw_main_arg,
             bgw_extra: RpgffiChar128::from(&builder.bgw_extra[..]).0,
             bgw_notify_pid: builder.bgw_notify_pid,
-        };
-
-        bgw
+        }
     }
 }
 
@@ -676,14 +711,15 @@ fn wait_latch(timeout: libc::c_long, wakeup_flags: WLflags) -> i32 {
     feature = "pg14",
     feature = "pg15",
     feature = "pg16",
-    feature = "pg17"
+    feature = "pg17",
+    feature = "pg18"
 ))]
 type RpgffiChar = RpgffiChar96;
 
 #[allow(dead_code)]
 struct RpgffiChar64([c_char; 64]);
 
-impl<'a> From<&'a str> for RpgffiChar64 {
+impl From<&str> for RpgffiChar64 {
     fn from(string: &str) -> Self {
         let mut r = [0; 64];
         for (dest, src) in r.iter_mut().zip(string.as_bytes()) {
@@ -695,7 +731,7 @@ impl<'a> From<&'a str> for RpgffiChar64 {
 
 struct RpgffiChar96([c_char; 96]);
 
-impl<'a> From<&'a str> for RpgffiChar96 {
+impl From<&str> for RpgffiChar96 {
     fn from(string: &str) -> Self {
         let mut r = [0; 96];
         for (dest, src) in r.iter_mut().zip(string.as_bytes()) {
@@ -707,7 +743,7 @@ impl<'a> From<&'a str> for RpgffiChar96 {
 
 struct RpgffiChar128([c_char; 128]);
 
-impl<'a> From<&'a str> for RpgffiChar128 {
+impl From<&str> for RpgffiChar128 {
     fn from(string: &str) -> Self {
         let mut r = [0; 128];
         for (dest, src) in r.iter_mut().zip(string.as_bytes()) {
@@ -720,7 +756,7 @@ impl<'a> From<&'a str> for RpgffiChar128 {
 #[allow(dead_code)]
 struct RpgffiChar1024([c_char; 1024]);
 
-impl<'a> From<&'a str> for RpgffiChar1024 {
+impl From<&str> for RpgffiChar1024 {
     fn from(string: &str) -> Self {
         let mut r = [0; 1024];
         for (dest, src) in r.iter_mut().zip(string.as_bytes()) {
