@@ -131,6 +131,7 @@ pub use pg_sys::{
     check_for_interrupts, debug1, debug2, debug3, debug4, debug5, ereport, error, function_name,
     info, log, notice, warning, FATAL, PANIC,
 };
+
 #[doc(hidden)]
 pub use pgrx_sql_entity_graph;
 
@@ -145,7 +146,7 @@ mod seal {
 //
 // Unless the compiling user explicitly told us that they're aware of this via `--features unsafe-postgres`.
 #[cfg(all(
-    any(feature = "pg15", feature = "pg16", feature = "pg17"),
+    any(feature = "pg15", feature = "pg16", feature = "pg17", feature = "pg18"),
     not(feature = "unsafe-postgres")
 ))]
 const _: () = {
@@ -175,15 +176,15 @@ const _: () = {
 /// <div class="example-wrap" style="display:inline-block">
 /// <pre class="ignore" style="white-space:normal;font:inherit;">
 ///
-/// **Note**: Every [`pgrx`][crate] extension **must** have this macro called at top level (usually `src/lib.rs`) to be valid.
+/// **Note**: Every [`pgrx`][crate] extension **must** have this macro called at top level (usually `src/lib.rs`) to be valid. You can use `pg_module_magic!()`, `pg_module_magic!(name, version)` or `pg_module_magic!(name = c"custom_name", version = c"1.0.0")`
 ///
 /// </pre></div>
 ///
 /// This calls [`pg_magic_func!()`](pg_magic_func).
 #[macro_export]
 macro_rules! pg_module_magic {
-    () => {
-        $crate::pg_magic_func!();
+    ($($key:ident $(= $value:expr)?),*) => {
+        $crate::pg_magic_func!($($key $(= $value)?),*);
 
         // A marker function which must exist in the root of the extension for proper linking by the
         // "pgrx_embed" binary during `cargo-pgrx schema` generation.
@@ -229,22 +230,28 @@ macro_rules! pg_module_magic {
 /// [Dynamic Loading]: https://www.postgresql.org/docs/current/xfunc-c.html#XFUNC-C-DYNLOAD
 #[macro_export]
 macro_rules! pg_magic_func {
-    () => {
-        #[no_mangle]
+    ($($key:ident $(= $value: expr)?),*) => {
+        #[unsafe(no_mangle)]
         #[allow(non_snake_case, unexpected_cfgs)]
         #[doc(hidden)]
         pub extern "C" fn Pg_magic_func() -> &'static ::pgrx::pg_sys::Pg_magic_struct {
-            static MY_MAGIC: ::pgrx::pg_sys::Pg_magic_struct = ::pgrx::pg_sys::Pg_magic_struct {
-                len: ::core::mem::size_of::<::pgrx::pg_sys::Pg_magic_struct>() as i32,
-                version: ::pgrx::pg_sys::PG_VERSION_NUM as i32 / 100,
-                funcmaxargs: ::pgrx::pg_sys::FUNC_MAX_ARGS as i32,
-                indexmaxkeys: ::pgrx::pg_sys::INDEX_MAX_KEYS as i32,
-                namedatalen: ::pgrx::pg_sys::NAMEDATALEN as i32,
-                #[cfg(feature = "pg12")]
-                float4byval: ::pgrx::pg_sys::USE_FLOAT4_BYVAL as i32,
-                float8byval: cfg!(target_pointer_width = "64") as i32,
-                #[cfg(any(feature = "pg15", feature = "pg16", feature = "pg17"))]
-                abi_extra: {
+            #[repr(transparent)]
+            struct AssertSync<T>(T);
+            unsafe impl<T> Sync for AssertSync<T> {}
+
+            // since Postgres calls this first, register our panic handler now
+            // so we don't unwind into C / Postgres
+            ::pgrx::pg_sys::panic::register_pg_guard_panic_hook();
+
+            static MY_MAGIC: AssertSync<::pgrx::pg_sys::Pg_magic_struct> = {
+                let len = ::core::mem::size_of::<::pgrx::pg_sys::Pg_magic_struct>() as i32;
+                let version = ::pgrx::pg_sys::PG_VERSION_NUM as i32 / 100;
+                let funcmaxargs = ::pgrx::pg_sys::FUNC_MAX_ARGS as i32;
+                let indexmaxkeys = ::pgrx::pg_sys::INDEX_MAX_KEYS as i32;
+                let namedatalen = ::pgrx::pg_sys::NAMEDATALEN as i32;
+                let float8byval = cfg!(target_pointer_width = "64") as i32;
+                #[cfg(not(any(feature = "pg13", feature = "pg14")))]
+                let abi_extra = {
                     // we'll use what the bindings tell us, but if it ain't "PostgreSQL" then we'll
                     // raise a compilation error unless the `unsafe-postgres` feature is set
                     let magic = ::pgrx::pg_sys::FMGR_ABI_EXTRA.to_bytes_with_nul();
@@ -255,15 +262,100 @@ macro_rules! pg_magic_func {
                         i += 1;
                     }
                     abi
-                },
+                };
+                let mut magic = ::pgrx::pg_sys::Pg_magic_struct {
+                    len,
+                    #[cfg(not(feature = "pg18"))]
+                    version,
+                    #[cfg(not(feature = "pg18"))]
+                    funcmaxargs,
+                    #[cfg(not(feature = "pg18"))]
+                    indexmaxkeys,
+                    #[cfg(not(feature = "pg18"))]
+                    namedatalen,
+                    #[cfg(not(feature = "pg18"))]
+                    float8byval,
+                    #[cfg(any(feature = "pg15", feature = "pg16", feature = "pg17"))]
+                    abi_extra,
+                    #[cfg(feature = "pg18")]
+                    abi_fields: ::pgrx::pg_sys::Pg_abi_values {
+                        version,
+                        funcmaxargs,
+                        indexmaxkeys,
+                        namedatalen,
+                        float8byval,
+                        abi_extra,
+                    },
+                    #[cfg(feature = "pg18")]
+                    name: ::core::ptr::null(),
+                    #[cfg(feature = "pg18")]
+                    version: ::core::ptr::null(),
+                };
+                #[allow(unused_macros)]
+                macro_rules! field_update {
+                    (name) => {
+                        field_update!(name = {
+                            const RAW: &str = env!("CARGO_PKG_NAME");
+                            const BUFFER: [u8; RAW.len() + 1] = {
+                                let mut buffer = [0u8; RAW.len() + 1];
+                                let mut i = 0_usize;
+                                while i < RAW.len() {
+                                    buffer[i] = RAW.as_bytes()[i];
+                                    i += 1;
+                                }
+                                buffer
+                            };
+                            const STR: &::core::ffi::CStr =
+                                if let Ok(s) = ::core::ffi::CStr::from_bytes_with_nul(&BUFFER) {
+                                    s
+                                } else {
+                                    panic!("there are null characters in CARGO_PKG_NAME")
+                                };
+                            const { STR }
+                        });
+                    };
+                    (version) => {
+                        field_update!(name = {
+                            const RAW: &str = env!("CARGO_PKG_VERSION");
+                            const BUFFER: [u8; RAW.len() + 1] = {
+                                let mut buffer = [0u8; RAW.len() + 1];
+                                let mut i = 0_usize;
+                                while i < RAW.len() {
+                                    buffer[i] = RAW.as_bytes()[i];
+                                    i += 1;
+                                }
+                                buffer
+                            };
+                            const STR: &::core::ffi::CStr =
+                                if let Ok(s) = ::core::ffi::CStr::from_bytes_with_nul(&BUFFER) {
+                                    s
+                                } else {
+                                panic!("there are null characters in CARGO_CRATE_VERSION")
+                             };
+                            const { STR }
+                        });
+                    };
+                    (name = $name:expr) => {
+                        let name: &'static ::core::ffi::CStr = $name;
+                        #[cfg(feature = "pg18")]
+                        {
+                            magic.name = ::core::ffi::CStr::as_ptr($name);
+                        }
+                    };
+                    (version = $version:expr) => {
+                        let version: &'static ::core::ffi::CStr = $version;
+                        #[cfg(feature = "pg18")]
+                        {
+                            magic.version = ::core::ffi::CStr::as_ptr($version);
+                        }
+                    };
+                }
+                $(field_update!($key $(= $value)?);)*
+                AssertSync(magic)
             };
 
-            // since Postgres calls this first, register our panic handler now
-            // so we don't unwind into C / Postgres
-            ::pgrx::pg_sys::panic::register_pg_guard_panic_hook();
-
             // return the magic
-            &MY_MAGIC
+            &MY_MAGIC.0
         }
     };
 }
