@@ -51,20 +51,20 @@ impl<T> PgLwLock<T> {
     }
 
     /// Obtain a shared lock (which comes with `&T` access)
-    pub fn share(&self) -> PgLwLockShareGuard<T> {
+    pub fn share(&self) -> PgLwLockShareGuard<'_, T> {
         unsafe {
             let shared = self.inner.get().read().as_ref().expect("PgLwLock was not initialized");
-            pg_sys::LWLockAcquire((*shared).lock_ptr, pg_sys::LWLockMode::LW_SHARED);
-            PgLwLockShareGuard { data: &*(*shared).data.get(), lock: (*shared).lock_ptr }
+            pg_sys::LWLockAcquire(shared.lock_ptr, pg_sys::LWLockMode::LW_SHARED);
+            PgLwLockShareGuard { data: &*shared.data.get(), lock: shared.lock_ptr }
         }
     }
 
     /// Obtain an exclusive lock (which comes with `&mut T` access)
-    pub fn exclusive(&self) -> PgLwLockExclusiveGuard<T> {
+    pub fn exclusive(&self) -> PgLwLockExclusiveGuard<'_, T> {
         unsafe {
             let shared = self.inner.get().read().as_ref().expect("PgLwLock was not initialized");
-            pg_sys::LWLockAcquire((*shared).lock_ptr, pg_sys::LWLockMode::LW_EXCLUSIVE);
-            PgLwLockExclusiveGuard { data: &mut *(*shared).data.get(), lock: (*shared).lock_ptr }
+            pg_sys::LWLockAcquire(shared.lock_ptr, pg_sys::LWLockMode::LW_EXCLUSIVE);
+            PgLwLockExclusiveGuard { data: &mut *shared.data.get(), lock: shared.lock_ptr }
         }
     }
 
