@@ -12,22 +12,27 @@
 
 NEW_VERSION=$1
 if [ -z "${NEW_VERSION}" ]; then
-	echo "Usage: ./prepare-release.sh NEW_VERSION_NUMBER"
+	echo "Usage: ./prepare-release.sh NEW_VERSION_NUMBER [base branch name]"
 	exit 1
 fi
+BASE_BRANCH=$2
+if [ -z "${BASE_BRANCH}" ]; then
+	echo "using develop as the base branch"
+	BASE_BRANCH=develop
+fi
 
-git switch develop
-git fetch origin
-git diff origin/develop | if [ "$0" = "" ]; then
-    echo "git diff found local changes on develop branch, cannot cut release."
+git switch ${BASE_BRANCH} || exit $?
+git fetch origin || exit $?
+git diff origin/${BASE_BRANCH} | if [ "$0" = "" ]; then
+    echo "git diff found local changes on ${BASE_BRANCH} branch, cannot cut release."
 elif [ "$NEW_VERSION" = "" ]; then
     echo "No version set. Are you just copying and pasting this without checking?"
 else
-    git pull origin develop --ff-only
-    git switch -c "prepare-${NEW_VERSION}"
+    git pull origin ${BASE_BRANCH} --ff-only || exit $?
+    git switch -c "prepare-${NEW_VERSION}" || exit $?
 
-    cargo install --path cargo-pgrx --locked
-    cargo pgrx init
+    cargo install --path cargo-pgrx --locked || exit $?
+    cargo pgrx init || exit $?
 
     # exit early if the script fails 
     ./update-versions.sh "${NEW_VERSION}" || exit $?
@@ -36,7 +41,7 @@ else
     # git diff -- . ':(exclude)Cargo.lock'
 
     # send it all to github
-    git commit -a -m "Update version to ${NEW_VERSION}"
-    git push --set-upstream origin "prepare-${NEW_VERSION}"
+    git commit -a -m "Update version to ${NEW_VERSION}" || exit $?
+    git push --set-upstream origin "prepare-${NEW_VERSION}" || exit $?
 fi
 
